#!/usr/bin/env python
# -*- Python -*-
## @pa10api
#
# Robot Arm(MHI-PA10) Resolved Motion Rate Control Components.

## @Library Source Path.
# To Be Editted by User.
RtmToolsDir="C:\\PA10RMRC\\pa10\\script"
## @Source Path.
# To Be Editted by User.
MyRtcDir="C:\\PA10RMRC\\pa10\\bin"

import sys
import time
import os
sys.path.append(".")
save_path = sys.path[:]
# To Be Editted by User.
sys.path.append(RtmToolsDir+'\\rtc_handle')
from rtc_handle import *
# To Be Editted by User.
sys.path.append('C:\\PA10RMRC\\pa10\\src\\tools\\geo')
# To Be Editted by User.
MyIP = "localhost:9876"
from geo import *
sys.path = save_path
# import stub files
import _GlobalIDL

##
#
def run_comp() :
  com_pa10vel ="START /MIN \"pa10vel\" " + MyRtcDir + "\\pa10velComp.exe -f " + MyRtcDir + "\\rtc.conf"
  os.popen2(com_pa10vel)
  com_mixer   ="START /MIN \"mixer\" " + MyRtcDir + "\\mixerComp.exe -f " + MyRtcDir + "\\rtc.conf"
  com_jinv    ="START /MIN \"jinv\" " + MyRtcDir + "\\jinvComp.exe -f " + MyRtcDir + "\\rtc.conf"
  com_frm_ctrl="START /MIN \"frm_ctrl\" " + MyRtcDir + "\\frm_ctrlComp.exe -f " + MyRtcDir + "\\rtc.conf"
  com_pa10fk  ="START /MIN \"pa10fk\" " + MyRtcDir + "\\pa10fkComp.exe -f " + MyRtcDir + "\\rtc.conf"
  os.popen2(com_mixer)
  os.popen2(com_jinv)
  os.popen2(com_frm_ctrl)
  os.popen2(com_pa10fk)
  print"  Wait for start up..",
  wait = 7
  while wait > 0 :
    print"%d" % (wait - 1),
    time.sleep(1)
    wait -= 1
  print" OK!"
##
#
def assign_variables(env) :
  global pa10fk, frm_ctrl, mixer, jinv, pa10vel
  pa10vel  = env.name_space[MyIP].rtc_handles['pa10vel0.rtc']
  pa10fk   = env.name_space[MyIP].rtc_handles['pa10fk0.rtc']
  frm_ctrl = env.name_space[MyIP].rtc_handles['frm_ctrl0.rtc']
  mixer    = env.name_space[MyIP].rtc_handles['mixer0.rtc']
  jinv     = env.name_space[MyIP].rtc_handles['jinv0.rtc']
  pa10fk.services['ComFk'].provided['com_fk'].narrow_ref(globals())
  global c_frame, m_tool_frm, m_target_frm
  c_frame=_GlobalIDL.Frame
  m_tool_frm = c_frame(MATRIX(),VECTOR())
  m_target_frm = c_frame(MATRIX(),VECTOR())
  global pa, pb, pc, pd, pe, aa, bb, cc
  pa = FRAME(xyzabc=[ 700.0, -300.0, 102.0, 0.0, pi, pi])
  pb = FRAME(xyzabc=[ 700.0, -300.0, 320.0, 0.0, pi, pi])
  pc = FRAME(xyzabc=[ 700.0,    0.0, 350.0, 0.0, pi, pi])
  pd = FRAME(xyzabc=[ 700.0,  300.0, 320.0, 0.0, pi, pi])
  pe = FRAME(xyzabc=[ 700.0,  300.0, 102.0, 0.0, pi, 0])
##
#
def connect_comp(env):
  env.con_list=[]
# 0
  con = IOConnector([pa10vel.outports['th'],pa10fk.inports['th']])
  env.con_list.append(con)
# 1
  con = IOConnector([pa10fk.outports['frame'],frm_ctrl.inports['cur_frm']])
  env.con_list.append(con)
# 3
  con = IOConnector([pa10fk.outports['jacob'],mixer.inports['jacob']])
  env.con_list.append(con)
# 4
  con = IOConnector([frm_ctrl.outports['vel'],mixer.inports['vel']])
  env.con_list.append(con)
# 5
  con = IOConnector([mixer.outports['j_list'],jinv.inports['y_jacob']])
  env.con_list.append(con)
# 6
  con = IOConnector([jinv.outports['vel'],pa10vel.inports['vel']])
  env.con_list.append(con)
  for con in env.con_list :
    con.connect()
##
#
def activate() :
  print"  Wait for activate..",
  i = 0
  lim = 14
  cnt = lim
  while i < lim :
    time.sleep(1)
    print"%d" % (cnt - 1),
    if(i == 0):
      pa10vel.activate()
    elif(i == 1):
      pa10fk.activate()
    elif(i == 6):
      frm_ctrl.activate()
    elif(i == 11):
      mixer.activate()
    i += 1
    cnt -= 1
  jinv.activate()
  time.sleep(0.5)
  print" OK!"
##
# initialization when this is imported.
env = RtmEnv(sys.argv,[MyIP])
raw_input("  PA10 is moving, OK?")
run_comp()
for ns in env.name_space :
  env.name_space[ns].list_obj()
assign_variables(env)
connect_comp(env)
##
#
def deactivate() :
  pa10vel.deactivate()
  pa10fk.deactivate()
  frm_ctrl.deactivate()
  mixer.deactivate()
  jinv.deactivate()
##
#
def pos(x, y, z, a, b, c) :
  return FRAME(xyzabc=[x, y, z, a, b, c])
##
#
def go_to(mv_rtc, pos) :
  m_target_frm.mat = pos.mat
  m_target_frm.pos = pos.vec
  mv_rtc.services['ComMv'].provided['com_move'].ref.move_to(m_target_frm)
#
def auto(mv_rtc, pos_list) :
  for pos in pos_list :
    go_to(mv_rtc, pos)
##
#
def set_tool(x, y, z) :
  m_tool_frm.pos[0] = x
  m_tool_frm.pos[1] = y
  m_tool_frm.pos[2] = z
  pa10fk.services['ComFk'].provided['com_fk'].ref.set_tool(m_tool_frm)
##
#
def end_comp() :
  raw_input("  Push STOP button. OK?")
  im_pa10vel= "taskkill /IM pa10velComp.exe"
  im_jinv   = "taskkill /IM jinvComp.exe"
  im_mixer  = "taskkill /IM mixerComp.exe"
  im_ctrl   = "taskkill /IM frm_ctrlComp.exe"
  im_pa10fk = "taskkill /IM pa10fkComp.exe"
  os.popen2(im_pa10vel)
  os.popen2(im_jinv)
  os.popen2(im_mixer)
  os.popen2(im_ctrl)
  os.popen2(im_pa10fk)
