// -*- C++ -*-
/*!
 * @file  jinv.cpp * @brief calc inv of Jacobian * $Date$ 
 *
 * $Id$ 
 */
#include "jinv.h"

// Module specification
// <rtc-template block="module_spec">
static const char* jinv_spec[] =
  {
    "implementation_id", "jinv",
    "type_name",         "jinv",
    "description",       "calc inv of Jacobian",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "jinv",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.n_size", "7",
    "conf.default.m_max", "100",
    ""
  };
// </rtc-template>

jinv::jinv(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_y_jacobIn("y_jacob", m_y_jacob),
    m_velOut("vel", m_vel)

    // </rtc-template>
{
  m_ejinv = new ExecJinv(this);
  m_y_jacobIn.setOnWrite(m_ejinv);
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("y_jacob", m_y_jacobIn);

  // Set OutPort buffer
  registerOutPort("vel", m_velOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

}

jinv::~jinv()
{
}


RTC::ReturnCode_t jinv::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("n_size", m_n_size, "7");
  bindParameter("m_max", m_m_max, "100");

  // </rtc-template>
  o_n_size = m_n_size;
  o_m_max = m_m_max;
  m_vel.data.length(o_n_size); // angular vel of o_n_size joints
  j_inv = new SVD(o_m_max,o_n_size);
  act_flag = 0;
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t jinv::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t jinv::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t jinv::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t jinv::onActivated(RTC::UniqueId ec_id)
{
  if((o_n_size != m_n_size) || (o_m_max != m_m_max)) {
    o_n_size = m_n_size;
    o_m_max = m_m_max;
    m_vel.data.length(o_n_size); // angular vel of o_n_size joints
    j_inv = new SVD(o_m_max,o_n_size);
  }
  act_flag = 1;
  return RTC::RTC_OK;
}

RTC::ReturnCode_t jinv::onDeactivated(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t jinv::onExecute(RTC::UniqueId ec_id)
{
  usleep(10000);
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t jinv::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t jinv::onError(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}

RTC::ReturnCode_t jinv::onReset(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t jinv::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t jinv::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void jinvInit(RTC::Manager* manager)
  {
    RTC::Properties profile(jinv_spec);
    manager->registerFactory(profile,
                             RTC::Create<jinv>,
                             RTC::Delete<jinv>);
  }
  
};



