// -*- C++ -*-
/*!
 * @file  jinv.h * @brief calc inv of Jacobian * @date  $Date$ 
 *
 * $Id$ 
 */
#ifndef JINV_H
#define JINV_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// svd class
#include "svd.h"

class ExecJinv;

using namespace RTC;

class jinv  : public RTC::DataFlowComponentBase
{
  friend class ExecJinv;

 public:
  jinv(RTC::Manager* manager);
  ~jinv();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  // virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  int m_n_size;
  int m_m_max;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedFloatSeq m_y_jacob;
  InPort<TimedFloatSeq> m_y_jacobIn;

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_vel;
  OutPort<TimedDoubleSeq, NullBuffer> m_velOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">

  // </rtc-template>

 private:
  int o_n_size;
  int o_m_max;
  ExecJinv* m_ejinv;
  SVD * j_inv;			//ْlNX̃CX^X
  int act_flag;
  int dummy;

};

class ExecJinv : public RTC::OnWrite<RTC::TimedFloatSeq>
{
  jinv* m_jinv;
public :
  ExecJinv(jinv* comp) : m_jinv(comp) {};
  void operator() (const RTC::TimedFloatSeq& value)
  {
    int i,j;
    int n_size, n_row;
    int row_size = m_jinv->o_n_size;
    int y_row = row_size +1;
    int m_max = m_jinv->o_m_max;
//    float tmp_jcb[row_size];
//    float tmp_vel[row_size];
    float *tmp_jcb, *tmp_vel;
	tmp_jcb = new float[row_size];
    tmp_vel = new float[row_size];

	//R|[lgĂꍇ
    if(m_jinv->act_flag==1) {
      n_size = value.data.length();
      n_row = n_size / y_row;
      //      std::cout << n_row << std::endl;
      if (n_row > m_max) {
	    std::cout << "number of row exceeds m_max." << std::endl;
        return;
      }
      m_jinv->j_inv -> set_m(n_row);
      if(n_row > 0) {
	    for(i=0;i<n_row;i++) {
	    // copying
	      for(j=0;j<row_size;j++) {
	        tmp_jcb[j]=value.data[y_row*i+1+j];
	      }
	      m_jinv->j_inv -> put_row_and_y(i, value.data[y_row*i], tmp_jcb);
	    }
		//ْlĊe̊px߂
	    m_jinv->j_inv -> do_svdcmp();
	    m_jinv->j_inv -> do_round_off(0.0001);
	    m_jinv->j_inv -> do_svbksb();
	    m_jinv->j_inv -> get_result(tmp_vel);  // copying
		for(i=0;i<row_size;i++) {
	      m_jinv->m_vel.data[i] = tmp_vel[i];  //copying
	    }
	  }
	  //pxf[^|[g(Out)֏
      m_jinv->m_velOut.write();
    }
	delete tmp_jcb;
	delete tmp_vel;
    //  usleep(1000);
    //  usleep(10000);
    //  return RTC::RTC_OK;
  };
};


extern "C"
{
  void jinvInit(RTC::Manager* manager);
};

#endif // JINV_H

