// -*- C++ -*-
/*!
 * @file  move.cpp
 * @brief forward kinema of PA10
 * $Date$
 *
 * $Id$
 */

#include "move.h"

// Module specification
// <rtc-template block="module_spec">
static const char* move_spec[] =
  {
    "implementation_id", "move",
    "type_name",         "move",
    "description",       "forward kinema of PA10",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "target_generator",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
	"conf.default.ref_value_x", "600",
    "conf.default.ref_value_y", "0",
    "conf.default.ref_value_z", "300",
    "conf.default.ref_value_a", "0",
    "conf.default.ref_value_b", "180",
    "conf.default.ref_value_c", "0",
    // System Configuration
    "corba.args", "",
    ""
  };
// </rtc-template>

move::move(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_stateIn("state", m_state),
    m_frameOut("frame", m_frame),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("state", m_stateIn);
  
  // Set OutPort buffer
  registerOutPort("frame", m_frameOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  m_frame.data.length(12);

}

move::~move()
{
}


RTC::ReturnCode_t move::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("ref_value_x", m_pos_x, "600");
  bindParameter("ref_value_y", m_pos_y, "0");
  bindParameter("ref_value_z", m_pos_z, "300");
  bindParameter("ref_value_a", m_rot_a, "0");
  bindParameter("ref_value_b", m_rot_b, "180");
  bindParameter("ref_value_c", m_rot_c, "0");

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t move::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t move::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t move::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t move::onActivated(RTC::UniqueId ec_id)
{

  // This component do not need any previous input.
  /*
  int i;
  i = 0;
  std::cout << "checking input" << std::endl;
  for(i=0;i<1000;i++) {
    if(not m_stateIn.isEmpty()) {
      return RTC::RTC_OK;
      std::cout << "ready" << std::endl;
    }
    usleep(1000);
  }
  std::cout << "lack of input" << std::endl;
  return RTC::RTC_ERROR;
  */

  int i,j;
  VECTOR tmp_vtr;
  MATRIX tmp_mtr;
  MATRIX tmp_amtr;
  MATRIX tmp_bmtr;
  MATRIX tmp_cmtr;
  REAL tmp_abc[3];
  /* mod.start 2009.9.18[error handling] by FSI */
  REAL vtrabc[6];		//͒l:x,y,z,a,b,c
						//ftHglƂĎĂ͒lFconfig[]
  float config[] = {m_pos_x, m_pos_y, m_pos_z, m_rot_a, m_rot_b, m_rot_c};
  int input_flag;		//1:L, 0:
  /* mod.end 2009.9.18[error handling] by FSI */

  // generate the first target

  /* mod.start 2009.9.18[error handling] by FSI */
  input_flag = 0;
  while(input_flag != 1){
	  std::cout << "Please input x,y,z [mm] and a,b,c [deg]: ";
	  // ͒l x,y,z,a,b,c
	  make_inputdata(vtrabc, config);
	  // ͒lK؂Ȓlǂ𔻒
	  if(can_move(vtrabc[0], vtrabc[1], vtrabc[2]) == 1){
		  //K؂łꍇ
		  input_flag = 1;
	  }else {
		  //sK؂łꍇ͍ē͂v
		  std::cout << "Out of movable area!" << std::endl;
	  }
  }

  for(i=0; i<3; i++) {
	tmp_vtr[i] = vtrabc[i];
	tmp_abc[i] = vtrabc[i + 3] * PI/180.0;	/* WA\L֕ϊ@[deg]  [rad] */
  }
  /* mod.end 2009.9.18[error handling] by FSI */
  std::cout << " vtr ";
  std::cout << tmp_vtr[0] << " ";
  std::cout << tmp_vtr[1] << " ";
  std::cout << tmp_vtr[2] << " ";
  std::cout << std::endl;
  /*for(i=0; i<3; i++) {
    std::cin >> tmp_abc[i];
    tmp_abc[i] *= PI/180.0;
  }*/
  std::cout << " abc ";
  std::cout << tmp_abc[0] << " ";
  std::cout << tmp_abc[1] << " ";
  std::cout << tmp_abc[2] << " ";
  std::cout << std::endl;
  //͒lڕWʒuEpf[^(FREME^)쐬
  ma(tmp_abc[0],tmp_amtr);
  mb(tmp_abc[1],tmp_bmtr);
  mc(tmp_abc[2],tmp_cmtr);
  m_mul(tmp_amtr,tmp_bmtr,tmp_mtr);
  m_mul(tmp_mtr,tmp_cmtr,tmp_mtr);
  for(i=0;i<3;i++) {
    for(j=0;j<3;j++) {
      m_frame.data[3*i+j]=tmp_mtr[i][j];
    }
    m_frame.data[9+i]=tmp_vtr[i];
  }

  std::cout << " write " << std::endl;
  //ڕWʒuEpf[^|[g(Out)֏
  m_frameOut.write();
  std::cout << " end " << std::endl;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t move::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t move::onExecute(RTC::UniqueId ec_id)
{

  int i,j;
  VECTOR tmp_vtr;
  MATRIX tmp_mtr;
  MATRIX tmp_amtr;
  MATRIX tmp_bmtr;
  MATRIX tmp_cmtr;
  REAL tmp_abc[3];
  /* mod.start 2009.9.18[error handling] by FSI */
  REAL vtrabc[6]; //input:x,y,z,a,b,c
  float config[] = {m_pos_x, m_pos_y, m_pos_z, m_rot_a, m_rot_b, m_rot_c};
  int input_flag;		//1:valid, 0:invalid
  /* mod.end 2009.9.18[error handling] by FSI */

  // after settled, generate the next target

  if(!m_stateIn.isEmpty()) {
    m_stateIn.read();
    if(m_state.data == 1) {  // settle
	  std::cout << "state = " << m_state.data << std::endl;
      /* mod.start 2009.9.18[error handling] by FSI */
	  input_flag = 0;
	  while(input_flag != 1){
		  std::cout << "Please input x,y,z [mm] and a,b,c [deg]: ";
		  // ͒l x,y,z,a,b,c
		  make_inputdata(vtrabc, config);
		  // ͒lK؂Ȓlǂ𔻒
		  if(can_move(vtrabc[0], vtrabc[1], vtrabc[2]) == 1){
			  input_flag = 1;
		  }else {
			  std::cout << "Out of movable area!" << std::endl;
		  }
	  }

	  for(i=0; i<3; i++) {
		tmp_vtr[i] = vtrabc[i];
		tmp_abc[i] = vtrabc[i + 3] * PI/180.0;	/* WA\L֕ϊ@[deg]  [rad] */
	  }
	  /* mod.end 2009.9.18[error handling] by FSI */

      /*for(i=0; i<3; i++) {
	std::cin >> tmp_vtr[i];
      }*/
      std::cout << " vtr ";
      std::cout << tmp_vtr[0] << " ";
      std::cout << tmp_vtr[1] << " ";
      std::cout << tmp_vtr[2] << " ";
      std::cout << std::endl;
      /*for(i=0; i<3; i++) {
	std::cin >> tmp_abc[i];
	tmp_abc[i] *= PI/180.0;
      }*/
      std::cout << " abc ";
      std::cout << tmp_abc[0] << " ";
      std::cout << tmp_abc[1] << " ";
      std::cout << tmp_abc[2] << " ";
      std::cout << std::endl;
	  //͒lڕWʒuEpf[^(FREME^)쐬
      ma(tmp_abc[0],tmp_amtr);
      mb(tmp_abc[1],tmp_bmtr);
      mc(tmp_abc[2],tmp_cmtr);
      m_mul(tmp_amtr,tmp_bmtr,tmp_mtr);
      m_mul(tmp_mtr,tmp_cmtr,tmp_mtr);
      for(i=0;i<3;i++) {
		for(j=0;j<3;j++) {
		  m_frame.data[3*i+j]=tmp_mtr[i][j];
		}
		m_frame.data[9+i]=tmp_vtr[i];
      }
	  std::cout << " write " << std::endl;
	  //ڕWʒuEpf[^|[g(Out)֏
	  m_frameOut.write();
	  std::cout << " end " << std::endl;
	}
  }else {
	  //DO NOTHING;
  }
  //  usleep(1000);
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t move::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t move::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t move::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t move::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t move::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/* add.start 2009.9.18 by FSI */
//֐@		 Fmake_inputdata@͒llł邩ǂmF
//1 float[]^Fvtrabc[6]@R\[͒l
//2 float[]^Fconfig[6]@ftHg͒l
//߂l		 FȂ
void make_inputdata(float vtrabc[6], float config[6]){

	int i,j;
	char buf;
	char str[100];

	i = 0;
	while(i<6){
		//͒llłȂꍇ
		if(scanf("%f", &vtrabc[i]) <= 0){
		  //ǂݍ񂾏ꍇ
		  if(scanf("%s", str) == 1){
			  //ftHg͒lw肷"default"Ƃ͂ꂽꍇ
			  if(strcmp(str, "default") == 0){
				  for(j=0; j<6; j++){
					  vtrabc[j] = config[j];
				  }
				  break;
			  }else {
				  //͂ǂݔ΂ē͂v
				  scanf("%*[^\n]");
				  scanf("%c", &buf);
				  std::cout << "Wrong input!\n" << std::endl;
				  std::cout << "Please input x,y,z [mm] and a,b,c [deg]: ";
			  }
		  //͒llłłȂꍇ
		  }else {
			  //͂ǂݔ΂ē͂v
			  scanf("%*[^\n]");
			  scanf("%c", &buf);
			  std::cout << "Wrong input!\n" << std::endl;
			  std::cout << "Please input x,y,z [mm] and a,b,c [deg]: ";
		  }
		  i = 0;
	  }//lǂݍ񂾏ꍇ
		else {
		  i++;	/* count the number of inputs */
	  }
	}
	scanf("%*[^\n]");
	scanf("%c", &buf);

}
//֐	   Fcan_move@͒l͈͓w肷̂ۂmF
//1 float^Fx@xW
//2 float^Fy@yW
//3 float^Fz@zW
//߂l  int^F1F͈͓,@0F͈͊O
int can_move(float x, float y, float z){

	if(x*x + y*y + (z - LB)*(z - LB) <= MOVABLE_RAD*MOVABLE_RAD){
		return 1;
	}else {
		return 0;
	}

}
/* add.end 2009.9.18 by FSI */

extern "C"
{

  void moveInit(RTC::Manager* manager)
  {
    RTC::Properties profile(move_spec);
    manager->registerFactory(profile,
                             RTC::Create<move>,
                             RTC::Delete<move>);
  }

};


