#!/usr/bin/env python
# -*- Python -*-

import sys
import time
sys.path.append(".")

# Import RTM module
import OpenRTM
import RTC

# Import Service implementation class
# <rtc-template block="service_impl">
from com_move_impl import *

# </rtc-template>

# Import Service stub modules
# <rtc-template block="global_idl">
import _GlobalIDL, _GlobalIDL__POA

# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
move_spec = ["implementation_id", "move", 
		 "type_name",         "move", 
		 "description",       "generate target point in task frame", 
		 "version",           "1.0", 
		 "vendor",            "Takashi Suehiro, AIST", 
		 "category",          "target_generator", 
		 "activity_type",     "DataFlowComponent", 
		 "max_instance",      "10", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 ""]

# </rtc-template>

class move(OpenRTM.DataFlowComponentBase):
	def __init__(self, manager):
		OpenRTM.DataFlowComponentBase.__init__(self, manager)

		# DataPorts initialization
		# <rtc-template block="data_ports">
		self._d_state = RTC.TimedLong(RTC.Time(0,0),0)
		self._stateIn = OpenRTM.InPort("state", self._d_state, OpenRTM.RingBuffer(8))
		self.registerInPort("state",self._stateIn)
		self._d_frame = RTC.TimedFloatSeq(RTC.Time(0,0),range(12))
		self._frameOut = OpenRTM.OutPort("frame", self._d_frame, OpenRTM.RingBuffer(8))
		self.registerOutPort("frame",self._frameOut)

		# </rtc-template>


		# DataPorts initialization
		# <rtc-template block="service_ports">
		self._ComMvPort = OpenRTM.CorbaPort("ComMv")
		self._com_move = ComMv_i(self)
		self._ComMvPort.registerProvider("com_move", "ComMv", self._com_move)
		self.registerPort(self._ComMvPort)

		# </rtc-template>


		# initialize of configuration-data.
		# <rtc-template block="configurations">

		# </rtc-template>

		 
	def onInitialize(self):
		# Bind variables and configuration variable
		# <rtc-template block="bind_config">


		# </rtc-template>
		return RTC.RTC_OK


	#def onFinalize(self):
	#	return RTC.RTC_OK

	#def onStartup(self, ec_id):
	#	return RTC.RTC_OK

	#def onShutdown(self, ec_id):
	#	return RTC.RTC_OK

	#def onActivated(self, ec_id):
	#	return RTC.RTC_OK

	#def onDeactivated(self, ec_id):
	#	return RTC.RTC_OK

	def onExecute(self, ec_id):
	        time.sleep(0.1)
		return RTC.RTC_OK

	#def onAborting(self, ec_id):
	#	return RTC.RTC_OK

	#def onError(self, ec_id):
	#	return RTC.RTC_OK

	#def onReset(self, ec_id):
	#	return RTC.RTC_OK

	#def onStateUpdate(self, ec_id):
	#	return RTC.RTC_OK

	#def onRateChanged(self, ec_id):
	#	return RTC.RTC_OK

def MyModuleInit(manager):
    profile = OpenRTM.Properties(defaults_str=move_spec)
    manager.registerFactory(profile,
                            move,
                            OpenRTM.Delete)

    # Create a component
    comp = manager.createComponent("move")



def main():
	mgr = OpenRTM.Manager.init(len(sys.argv), sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()


