// -*- C++ -*-
/*!
 * @file  pa10fk.cpp
 * @brief forward kinema of PA10
 * $Date$
 *
 * $Id$
 */

#include "pa10fk.h"

// Module specification
// <rtc-template block="module_spec">
static const char* pa10fk_spec[] =
  {
    "implementation_id", "pa10fk",
    "type_name",         "pa10fk",
    "description",       "forward kinema of PA10",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "fk_7dof",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
	// mod.start 2009.9.15 by FSI
	"conf.default.tool_x", "0.0",
    "conf.default.tool_y", "0.0",
    "conf.default.tool_z", "150.0",
    "conf.default.tool_a", "0.0",
    "conf.default.tool_b", "0.0",
    "conf.default.tool_c", "0.0",
	// mod.end 2009.9.15 by FSI

    ""
  };
// </rtc-template>

pa10fk::pa10fk(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_thIn("th", m_th),
    m_frameOut("frame", m_frame),
    m_jacobOut("jacob", m_jacob),
    m_ComFkPort("ComFk"),
    //    m_fk(this),
    // </rtc-template>
	dummy(0)
{
  m_fk = new ExecFk(this);
  m_thIn.setOnWrite(m_fk);
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("th", m_thIn);
  
  // Set OutPort buffer
  registerOutPort("frame", m_frameOut);
  registerOutPort("jacob", m_jacobOut);
  
  // Set service provider to Ports
  m_ComFkPort.registerProvider("com_fk", "ComFk", m_com_fk);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_ComFkPort);
  
  // </rtc-template>

  m_frame.data.length(12); // 3*3+3
  m_jacob.data.length(42);   // 6*7

}

pa10fk::~pa10fk()
{
}


RTC::ReturnCode_t pa10fk::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  //mod:start 2009.9.15 by FSI
  bindParameter("tool_x", m_tool_x, "0.0");
  bindParameter("tool_y", m_tool_y, "0.0");
  bindParameter("tool_z", m_tool_z, "150.0");
  bindParameter("tool_a", m_tool_a, "0.0");
  bindParameter("tool_b", m_tool_b, "0.0");
  bindParameter("tool_c", m_tool_c, "0.0");
  //mod:end 2009.9.15 by FSI
  // </rtc-template>

  act_flag = 0;
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t pa10fk::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t pa10fk::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t pa10fk::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t pa10fk::onActivated(RTC::UniqueId ec_id)
{
  //mod:start 2009.9.15 by FSI 
  //FRAME tmp;
  //MATRIX a_mat[8];
  //tmp.vtr[0] = 0.0;
  //tmp.vtr[1] = 0.0;
  //tmp.vtr[2] = 150.0;
  //for(int i=0;i<3;i++) {
  //  for(int j=0;j<3;j++) {
  //    a_mat[7][i][j] = 0.0;
  //  }
  //  a_mat[7][i][i] = 1.0;
  //}
  //m_ass(tmp.mtr, a_mat[7]);
  FRAME tmp;
  MATRIX tmp_mtr;
  MATRIX tmp_amtr;
  MATRIX tmp_bmtr;
  MATRIX tmp_cmtr;

  //c[߂
  tmp.vtr[0] = m_tool_x;
  tmp.vtr[1] = m_tool_y;
  tmp.vtr[2] = m_tool_z;

  //c[̎p߂
  m_tool_a = m_tool_a * PI/180.0;	/* WA\L֕ϊ [deg]  [rad] */
  m_tool_b = m_tool_b * PI/180.0;
  m_tool_c = m_tool_c * PI/180.0;

  //c[̎p\FRAME^f[^𐶐
  ma(m_tool_a,tmp_amtr);
  mb(m_tool_b,tmp_bmtr);
  mc(m_tool_c,tmp_cmtr);
  m_mul(tmp_amtr,tmp_bmtr,tmp_mtr);
  m_mul(tmp_mtr,tmp_cmtr,tmp_mtr);
  for(int i=0;i<3;i++) {
    for(int j=0;j<3;j++) {
      tmp.mtr[i][j] = tmp_mtr[i][j];
    }
  }
  //mod:end 2009.9.15 by FSI
  //c[̒Ǝpݒ肷
  m_com_fk.f_kinema.set_tool(&tmp);
  act_flag = 1;
  return RTC::RTC_OK;
/*  int i;
  i = 0;
  std::cout << "checking input" << std::endl;
  for(i=0;i<1000;i++) {
    if(not m_thIn.isEmpty()) {
      std::cout << "ready" << std::endl;
      return RTC::RTC_OK;
    }
    usleep(1000);
  }
  std::cout << "lack of input" << std::endl;
  return RTC::RTC_ERROR;
*/
}


RTC::ReturnCode_t pa10fk::onDeactivated(RTC::UniqueId ec_id)
{
  act_flag = 0;

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t pa10fk::onExecute(RTC::UniqueId ec_id)
{

	
  int i,j;
  FRAME tmp_frm;
  Jacob tmp_jcb;
  REAL tmp_th[7];
  REAL tmp_abc[3];

  if(not m_thIn.isEmpty()) {
    //    std::cout << "pre read" << std::endl;
    m_thIn.read();
    //    std::cout << "read" << std::endl;
    for(i=0;i<7;i++) tmp_th[i]=m_th.data[i];
    m_com_fk.f_kinema.calc_fk(tmp_th);
    m_com_fk.f_kinema.link_frame(7, &tmp_frm);
    for(i=0;i<3;i++) {
      for(j=0;j<3;j++) {
	m_frame.data[3*i+j]=tmp_frm.mtr[i][j];
      }
      m_frame.data[9+i]=tmp_frm.vtr[i];
    }
    m_com_fk.f_kinema.jacob(tmp_jcb);
    for(i=0;i<6;i++)
      for(j=0;j<7;j++)
	m_jacob.data[7*i+j]=tmp_jcb[i][j];
    //    std::cout << "pre write" << std::endl;
    m_frameOut.write();
    m_jacobOut.write();
    mabc(tmp_frm.mtr, tmp_abc);
    //    std::cout << "write" << std::endl;
  }
  //  usleep(1000);
  usleep(10000);
  
  return RTC::RTC_OK;
  
}
*/

/*
RTC::ReturnCode_t pa10fk::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t pa10fk::onError(RTC::UniqueId ec_id)
{ 
  act_flag = 0;
  return RTC::RTC_OK;
}

RTC::ReturnCode_t pa10fk::onReset(RTC::UniqueId ec_id)
{
  act_flag = 0;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t pa10fk::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t pa10fk::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{

  void pa10fkInit(RTC::Manager* manager)
  {
    RTC::Properties profile(pa10fk_spec);
    manager->registerFactory(profile,
                             RTC::Create<pa10fk>,
                             RTC::Delete<pa10fk>);
  }

};


