// -*- C++ -*-
/*!
 * @file  pa10vel.cpp * @brief velocity controlled PA10 arm * $Date$ 
 *
 * $Id$ 
 */
#include "pa10vel.h"

// Module specification
// <rtc-template block="module_spec">
static const char* pa10vel_spec[] =
  {
    "implementation_id", "pa10vel",
    "type_name",         "pa10vel",
    "description",       "velocity controlled PA10 arm",
    "version",           "1.0",
    "vendor",            "FSI",
    "category",          "pa10",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>

pa10vel::pa10vel(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_velIn("vel", m_vel),
    m_thOut("th", m_th),
    m_ComPA10Port("ComPA10")

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("vel", m_velIn);

  // Set OutPort buffer
  registerOutPort("th", m_thOut);

  // Set service provider to Ports
  m_ComPA10Port.registerProvider("com_pa10", "ComPA10", m_com_pa10);

  // Set service consumers to Ports

  // Set CORBA Service Ports
  registerPort(m_ComPA10Port);

  int i;

  m_vel.data.length(7);
  m_th.data.length(7);

  for(i=0;i<7;i++) {
     ref_vel[i]=0.0;
     com_vel[i]=0.0;
  }

  // </rtc-template>

}

pa10vel::~pa10vel()
{
}

//-------------------------------------------------------------
// helper functions
//-------------------------------------------------------------
float calc_decay(float d_ftm, float l_ftm, float t_cnst) {
  if(d_ftm > l_ftm) {
    return exp((l_ftm - d_ftm)/ t_cnst);
  } else {
    return 1.0;
  }
}

RTC::ReturnCode_t pa10vel::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

  long err;
  std::cout << "start pa10" << std::endl;
  //ҋ@pƂĎw肳ꂽpx֊es
  err = m_com_pa10.init_pa10();
  std::cout << "ready" << std::endl;
  if( err != ERR_OK ){
      return RTC::RTC_ERROR;
  }
  // </rtc-template>
  return RTC::RTC_OK;
}


RTC::ReturnCode_t pa10vel::onFinalize()
{
  //PACȕIyуA[̃N[Ys
  m_com_pa10.end_pa10();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t pa10vel::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t pa10vel::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t pa10vel::onActivated(RTC::UniqueId ec_id)
{
  int i;
  long err;

//  std::cout << "on activated pa10" << std::endl;

  //ݎ擾
  nTime = timeGetTime();
  oTime = nTime;

  blank_count = 0;
  d_fact = 1.0;

  for(i=0;i<7;i++) {
    ref_vel[i]=0.0;
    com_vel[i]=0.0;
  }
  //PA10̊ex̊Jns
  err =  m_com_pa10.on_pa10();
  if(err != ERR_OK) {
    //PA10̊eẍꎞ~s
    err =  m_com_pa10.off_pa10();
    return RTC::RTC_ERROR;
  }

  return RTC::RTC_OK;
}

RTC::ReturnCode_t pa10vel::onDeactivated(RTC::UniqueId ec_id)
{
  //PA10̊eẍꎞ~s
  m_com_pa10.off_pa10();

  return RTC::RTC_OK;
}

RTC::ReturnCode_t pa10vel::onExecute(RTC::UniqueId ec_id)
{
  int i;
  long err;
  ANGLE an;

//  std::cout << "on execute" << std::endl;
  nTime = timeGetTime();
  std::cout << "actual delta time(ms): " << (nTime - oTime) << std::endl;
  oTime = nTime;

  if( m_com_pa10.m_act_flg == 1 ){
    for(i=0;i<7;i++) {
      ref_vel[i]=0.0;
      com_vel[i]=0.0;
    }
  } else {
    if(m_velIn.isEmpty()) {
//      std::cout << "m_velIn Empty" << std::endl;
      blank_count++;
      di_tm = blank_count*(float)TIMESTEP;
      d_fact = calc_decay(di_tm, (float)0.1, (float)1.0);  // apply decay
    } else {
//      std::cout << "m_velIn Not Empty" << std::endl;
      blank_count = 0;
      m_velIn.read();

      for(i=0;i<7;i++) {
		//֐ߊpxvZ̑傫\tgEFA~bg𒴂ȂmF
        if ( fabs(m_vel.data[i]) <= m_com_pa10.m_vel_lim[i] ){
          ref_vel[i]=(float)m_vel.data[i];
        } else {
          printf( "over limit[%d]:[%f] > [%f]\n", i, m_vel.data[i] , m_com_pa10.m_vel_lim[i] );
          ref_vel[i]=0;
        }
      }
      d_fact = 1.0;
    }
    for(i=0;i<7;i++) {
      com_vel[i]=d_fact*ref_vel[i];
//      std::cout << " " << com_vel[i];
    }
//    std::cout << std::endl;

    err = pa_odr_vel(ARM0, com_vel);
//    std::cout << "pa_odr_vel" << " " << err << std::endl;
    if(err != ERR_OK) {
      std::cout << "pa_odr_vel" << " " << err << std::endl;
      if(err > -1000 || err <= -2000) { // not warning
		//PA10̊eẍꎞ~s
        err =  m_com_pa10.off_pa10();
        return RTC::RTC_ERROR;
      }
    }

    err = pa_get_agl(ARM0, &an);
//  std::cout << "pa_get_agl" << " " << err << std::endl;

	//֐ߊpxȏꍇ͂f[^|[g(Out)֏
    if(err == ERR_OK) {
      m_th.data[0] =  an.s1;
      m_th.data[1] =  an.s2;
      m_th.data[2] =  an.s3;
      m_th.data[3] =  an.e1;
      m_th.data[4] =  an.e2;
      m_th.data[5] =  an.w1;
      m_th.data[6] =  an.w2;
//    std::cout << " " << m_th.data[0];
//    std::cout << " " << m_th.data[1];
//    std::cout << " " << m_th.data[2];
//    std::cout << " " << m_th.data[3];
//    std::cout << " " << m_th.data[4];
//    std::cout << " " << m_th.data[5];
//    std::cout << " " << m_th.data[6];
//    std::cout << std::endl;
      m_thOut.write();
    }
  }

 
//  usleep(1000);

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t pa10vel::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t pa10vel::onError(RTC::UniqueId ec_id)
{
  long err;

  //PACȕyуA[̃Zbgs
  err =  m_com_pa10.reset_pa10();
  if(err != ERR_OK) {
    if(err > -1000 || err <= -2000) { // not warning
	  //PA10̊eẍꎞ~s
      err =  m_com_pa10.off_pa10();
    }
  }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t pa10vel::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t pa10vel::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t pa10vel::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void pa10velInit(RTC::Manager* manager)
  {
    RTC::Properties profile(pa10vel_spec);
    manager->registerFactory(profile,
                             RTC::Create<pa10vel>,
                             RTC::Delete<pa10vel>);
  }
  
};



