// -*- C++ -*-
/*!
 * @file  vel_7dof.cpp
 * @brief simulator of velocity controlled 7dof arm
 * $Date$
 *
 * $Id$
 */

#include "vel_7dof.h"


// Module specification
// <rtc-template block="module_spec">
static const char* vel_7dof_spec[] =
  {
    "implementation_id", "vel_7dof",
    "type_name",         "vel_7dof",
    "description",       "simulator of velocity controlled 7dof arm",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "vel_7dof",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
	/* add.start 2009.9.15 by FSI */
	"conf.default.Th0", "0.0",
    "conf.default.Th1", "0.5",
    "conf.default.Th2", "0.0",
    "conf.default.Th3", "1.6",
    "conf.default.Th4", "0.0",
    "conf.default.Th5", "1.0",
    "conf.default.Th6", "0.0",
	/* add.end 2009.9.15 by FSI */
    ""
  };
// </rtc-template>

vel_7dof::vel_7dof(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_velIn("vel", m_vel),
    m_thOut("th", m_th),
    m_th2Out("th2", m_th2),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("vel", m_velIn);
  
  // Set OutPort buffer
  registerOutPort("th", m_thOut);
  registerOutPort("th2", m_th2Out);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  m_th.data.length(7);
  m_th2.data.length(7);

  
  int i;

  for(i=0;i<7;i++) {
    vel[i]=0.0;
    ref_vel[i]=0.0;
    th[i]=0.0;
  }
  //add.start 2009.10.13 by Kojima
  //PA10֐ߊp̃\tgEFA~bgݒ
  max_th[0] = 177 * PI/180.0;
  max_th[1] = 94 * PI/180.0;
  max_th[2] = 174 * PI/180.0;
  max_th[3] = 137 * PI/180.0;
  max_th[4] = 255 * PI/180.0;
  max_th[5] = 165 * PI/180.0;
  max_th[6] = 255 * PI/180.0;
  //add.end 2009.10.13 by Kojima

}

vel_7dof::~vel_7dof()
{
}

//-------------------------------------------------------------
// helper functions
//-------------------------------------------------------------
float calc_tm_dif(RTC::Time tm1, RTC::Time tm2) {
  return (float)(tm1.sec - tm2.sec)
    + (float)(1000000000 + tm1.nsec - tm2.nsec)/ 1000000000.0 - 1.0;
}
//
float calc_decay(float d_ftm, float l_ftm, float t_cnst) {
  if(d_ftm > l_ftm) {
    return exp((l_ftm - d_ftm)/ t_cnst);
  } else {
    return 1.0;
  }
}

RTC::ReturnCode_t vel_7dof::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  /* add.start 2009.9.15 by FSI */
  bindParameter("Th0", m_Th0, "0.0");
  bindParameter("Th1", m_Th1, "0.5");
  bindParameter("Th2", m_Th2, "0.0");
  bindParameter("Th3", m_Th3, "1.6");
  bindParameter("Th4", m_Th4, "0.0");
  bindParameter("Th5", m_Th5, "1.0");
  bindParameter("Th6", m_Th6, "0.0");
  /* add.end 2009.9.15 by FSI */

  // </rtc-template>


  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t vel_7dof::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t vel_7dof::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t vel_7dof::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t vel_7dof::onActivated(RTC::UniqueId ec_id)
{
  int i;
  timeval time_val;

  std::cout << "baka 0" << std::endl;

  //  tim.new_time(n_tm.sec, n_tm.nsec);
/*
  gettimeofday(&time_val, NULL);
  n_tm.sec = time_val.tv_sec;
  n_tm.nsec = time_val.tv_usec * 1000;
  std::cout << "baka 1" << std::endl;

  o_tm.sec = n_tm.sec;
  o_tm.nsec = n_tm.nsec;
  in_tm.sec = n_tm.sec;
  in_tm.nsec = n_tm.nsec;
*/

  nTime  = timeGetTime();
  oTime  = nTime;
  inTime = nTime;

  d_fact = 1.0;
  //   ready();
  /* add.start 2009.9.15 by FSI */
  th[0] =  m_Th0;
  th[1] =  m_Th1;
  th[2] =  m_Th2;
  th[3] =  m_Th3;
  th[4] =  m_Th4;
  th[5] =  m_Th5;
  th[6] =  m_Th6;
  /* add.end 2009.9.15 by FSI */
  std::cout << "baka 2" << std::endl;
  for(i=0;i<7;i++) {
    vel[i]=0.0;
    ref_vel[i]=0.0;
    m_th.data[i]=th[i];
    m_th2.data[i]=th[i];
  }
  std::cout << "baka 3" << std::endl;
  m_thOut.write();
  m_th2Out.write();
  std::cout << "baka 4" << std::endl;
  usleep(1000);
  std::cout << "baka 5" << std::endl;
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t vel_7dof::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t vel_7dof::onExecute(RTC::UniqueId ec_id)
{
  int i;
  timeval time_val;

  //  std::cout << "aho 0" << std::endl;

  //  tim.new_time(n_tm.sec, n_tm.nsec);

//  gettimeofday(&time_val, NULL);
//  n_tm.sec = time_val.tv_sec;
//  n_tm.nsec = time_val.tv_usec * 1000;

//  d_tm = calc_tm_dif(n_tm, o_tm);  // delta time
//  o_tm.sec = n_tm.sec;
//  o_tm.nsec = n_tm.nsec;

  //݂̎擾
  nTime = timeGetTime();
  //OonExecuteĂяǒoߎԂ߂
  d_tm = (nTime - oTime)*0.001;
  std::cout << "actual delta time(ms): " << d_tm*1000 << std::endl;
  oTime = nTime;

  //  std::cout << "aho 1" << std::endl;
  if(m_velIn.isEmpty()) {
    //    std::cout << "aho 2" << std::endl;
//    di_tm = calc_tm_dif(n_tm,in_tm);
    di_tm = (nTime - inTime)*0.001;
    std::cout << di_tm << std::endl;
    d_fact = calc_decay(di_tm, 0.1, 1.0);  // apply decay
  } else {
    //    std::cout << "aho 3" << std::endl;
//    in_tm.sec = n_tm.sec;
//    in_tm.nsec = n_tm.nsec;
    inTime = nTime;
    m_velIn.read();
    for(i=0;i<7;i++) {
      ref_vel[i]=m_vel.data[i];
    }
    d_fact = 1.0;
  }
  //d_tm̊֐ߊpxvZ :d_tm P[s]
  for(i=0;i<7;i++) {
    vel[i]=d_fact*ref_vel[i];
    th[i] += vel[i]*d_tm;
	//mod.start 2009.10.13 by FSI
	//߂֐ߊpx̑傫\tgEFA~bg𒴂ȂmF
	if(th[i] > max_th[i]){
		th[i] = max_th[i];
	}else if(th[i] < -1*max_th[i]){
		th[i] = -1*max_th[i];
	}else {
		;//DO NOTHING
	}
	printf("th[%d] = %f\n", i, th[i]);
	//mod.end 2009.10.13 by FSI
    m_th.data[i]=th[i];
    m_th2.data[i]=th[i];
  }
  //֐ߊpxf[^|[g(Out)֏
  m_thOut.write();
  m_th2Out.write();
  //  std::cout << m_th.data.length() << std::endl;
  Sleep(0.1);
//  usleep(10000);
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t vel_7dof::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t vel_7dof::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t vel_7dof::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t vel_7dof::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t vel_7dof::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{

  void vel_7dofInit(RTC::Manager* manager)
  {
    RTC::Properties profile(vel_7dof_spec);
    manager->registerFactory(profile,
                             RTC::Create<vel_7dof>,
                             RTC::Delete<vel_7dof>);
  }

};


