// -*- C++ -*-
/*!
 * @file  vel_7dof.h
 * @brief simulator of velocity controlled 7dof arm
 * @date  $Date$
 *
 * $Id$
 */

#ifndef VEL_7DOF_H
#define VEL_7DOF_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include <windows.h>
#include <mmsystem.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// #include <time_measure.h>
#include <math.h>
#include <geo_std.h>

using namespace RTC;

class vel_7dof
  : public RTC::DataFlowComponentBase
{
 public:
  vel_7dof(RTC::Manager* manager);
  ~vel_7dof();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry()
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
 virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  // virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
 virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /* add.start 2009.9.15 by FSI */
  /*!
   * 
   * - Name: Th0 Th0
   * - DefaultValue: conf_default
   */
  float m_Th0;
  /*!
   * 
   * - Name: Th1 Th1
   * - DefaultValue: conf_default
   */
  float m_Th1;
  /*!
   * 
   * - Name: Th2 Th2
   * - DefaultValue: conf_default
   */
  float m_Th2;
  /*!
   * 
   * - Name: Th3 Th3
   * - DefaultValue: conf_default
   */
  float m_Th3;
  /*!
   * 
   * - Name: Th4 Th4
   * - DefaultValue: conf_default
   */
  float m_Th4;
  /*!
   * 
   * - Name: Th5 Th5
   * - DefaultValue: conf_default
   */
  float m_Th5;
  /*!
   * 
   * - Name: Th6 Th6
   * - DefaultValue: conf_default
   */
  float m_Th6;
  /* add.start 2009.9.15 by FSI */
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedDoubleSeq m_vel;
  InPort<TimedDoubleSeq> m_velIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_th;
  OutPort<TimedDoubleSeq, NullBuffer> m_thOut;
  TimedDoubleSeq m_th2;
  OutPort<TimedDoubleSeq, NullBuffer> m_th2Out;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  //  TimeMeasure tim;
//  RTC::Time n_tm, o_tm;
//  RTC::Time in_tm;
  DWORD oTime, nTime, inTime;
  float d_tm, di_tm, d_fact;
  float ref_vel[7];
  float th[7],vel[7];
  //add.start 2009.10.13 by Kojima
  float max_th[7];
  //add.end 2009.10.13 by Kojima

  int dummy;

};


extern "C"
{
  void vel_7dofInit(RTC::Manager* manager);
};

#endif // VEL_7DOF_H
